#!/bin/sh

SD_MOUNTPATH="/data"
ETC_BACKUP="${SD_MOUNTPATH}/etc"
NET_BACKUP="${ETC_BACKUP}/network"
PARTITION_1="/dev/mmcblk0p1"
PARTITION_2="/dev/mmcblk0p2"

#
# check if mmcblk0p1 is mounted on /data
#
is_partition1_mounted(){

    if ! cat < "/proc/mounts" | grep "$PARTITION_1" >/dev/null 2>&1
    then
        return 1
    fi
    return 0
}

#
# create weblog file structure
#
create_filestructure() {
    echo "Creating directory-structure in <${SD_MOUNTPATH}> ..."
    [ -d ${SD_MOUNTPATH}/log ] || mkdir -p ${SD_MOUNTPATH}/log
    [ -d ${SD_MOUNTPATH}/update ] || mkdir -p ${SD_MOUNTPATH}/update
    [ -d ${SD_MOUNTPATH}/db ] || mkdir -p ${SD_MOUNTPATH}/db
    [ -d ${SD_MOUNTPATH}/etc ] || mkdir -p ${SD_MOUNTPATH}/etc
    [ -d ${SD_MOUNTPATH}/tmp ] || mkdir -p  ${SD_MOUNTPATH}/tmp
    echo "Dircetory-structure <${SD_MOUNTPATH}> created"
}

#
# Prepare the sd-card
#
sd_card_is_a_weblog_data_card() {
    echo "Checking for a sd-card containing weblog-data ..."    
    if test ! -b "$PARTITION_1"
    then
        echo "No partitions found on sd-card --> this is not a weblog data-card."
        return 1
    fi

    if test -b "$PARTITION_2"
    then
        echo "More than one partition found on sd-card --> this is not a weblog data-card."
        return 1
    fi

    PARTITION_TYPE=$(lsblk -f "$PARTITION_1"|tail -1 | tr -s [:blank:] |cut -d' ' -f 2)
    if test "$PARTITION_TYPE" != "ext3"
    then
        echo "Partition on sd-card is not formatted using ext3 --> this is not a weblog data-card."
        return 1
    fi

    echo "The sd-card is a valid weblog data-card."
    return 0
}

make_weblog_data_card() {
    if test is_partition1_mounted
    then
        umount "$PARTITION_1"
    fi

    echo "Creating partition-table on </dev/mmcblk0> ..."
    fdisk /dev/mmcblk0 >/dev/null 2>&1 <<EOF
n
p
1


w
EOF

    # Once Partitions are created, systems likes to mount automatically
    umount "$PARTITION_1" || true

    echo "Creating ext3-filesystem on <${PARTITION_1}> ..."
    if ! mkfs.ext3 "${PARTITION_1}"
    then
        echo "Failed to create filesystem on <${PARTITION_1}>!"
        return 1
    fi

    echo "The sd-card is now prepared as a weblog data-card."
    return 0
}

#
# restore timezone-settings from sd-card to system
#
restore_timezone(){
    if [ -L "${ETC_BACKUP}/localtime" ]
    then
        if ! cp -P "${ETC_BACKUP}/localtime" "/etc/localtime"
        then
            echo "Failed to restore timezone!"
            return 1
        fi
    fi
    return 0
}

#
# apply network-config from sd-card to system
#
restore_network_config (){
    if [ -x "/lib/systemd/systemd" ]
    then
        # TODO:
        echo " TODO: Restore network-configuration for systemd!"
        return 1
    else
        if [ -d "${NET_BACKUP}" ]
        then
            echo "Restoring configuration from /data/etc/..."
            [ -f "${NET_BACKUP}/interfaces" ] && cp -df "${NET_BACKUP}/interfaces" /etc/network/
            [ -f "${NET_BACKUP}/wpa_supplicant.conf" ] && cp -df "${NET_BACKUP}/wpa_supplicant.conf" /etc/wpa_supplicant.conf
            [ -f "${NET_BACKUP}/wpa" ] && cp -df "${NET_BACKUP}/wpa" /etc/default/wpa
            echo "Restoring configuration from /data/etc/... done"

        fi
    fi

    return 0
}

##################  main ###################

if ! systemctl --version 1>/dev/null 2>/dev/null
then
    # create the mountpoint for SD-Card onc /data
    [ -d "$SD_MOUNTPATH" ] || mkdir -p "$SD_MOUNTPATH"

    if [ -b /dev/mmcblk0 ]
    then
        if ! sd_card_is_a_weblog_data_card
        then
            if ! make_weblog_data_card
            then
                exit 1
            fi
        fi
        
        # mount the sd-card
        mount -t ext3 -o noatime,nodiratime,rw,data=ordered "$PARTITION_1" "$SD_MOUNTPATH"/ || echo "Failed to mount <${PARTITION_1}> to <${SD_MOUNTPATH}>!"
    fi
    
    # create symblolic Link for the correct mbus configuration
    cd /etc/mbus/ && /bin/ln -sf  main-WEBLOG250-TX28.conf   main.conf
    # create data-structure on SD-Card
    create_filestructure
    # restore settings only for SYS V
    restore_timezone || true
    restore_network_config || true
else
     # create symblolic Link for the correct mbus configuration
    device_type=$(grep compatible /etc/rauc/system.conf | cut -d'=' -f2| cut -d'-' -f 1)
    if test "$device_type" == "weblog250"
    then
        cd /etc/mbus/ && /bin/ln -sf  main-WEBLOG250-TX8M.conf   main.conf
    else
        cd /etc/mbus/ && /bin/ln -sf  main-WEBLOG120-TX8M.conf   main.conf
    fi

    create_filestructure
fi
