#!/bin/sh
#===============================================================================
# This script checks whether an FTP- or EMAL-server is reachable and can be used.
# return 0 in case of success and 1 otherwise
#
# Input parameter are depends on the given type in $1(FTP or EMAIL):
# mandatory parameter are marked with (M) other parameter are optional.
# All parameter must be given at least optional parameter can be filled
# with none.
# FTP   ==> TYPE USERNAME PASSWORD SERVER PORT SUBDIR SECUREFLAG
#   TYPE    : type of scheduler FTP (M)
#   USERNAME: ftp user name
#   PASSWORD: user password
#   SERVER  : ftp Server (M)
#   PORT    : ftp server port (M)
#   SUBDIR  : subdirectory to write into
#   SECURE  : flag to use ftp or secure ftp 'sftp'
# EMALE ==> TOADDRESS FROMADDRESS  SERVER USERNAME PASSWD SECUREFLAG
#   TYPE       : type of scheduler EMAIL (M)
#   TOADDRESS  : (M) semicolon separated list of email recipient e.g. x@gmail.com
#   FROMADDRESS: email sender  lala@gmail.com
#   SERVER     : email server (M)
#   USERNAME   : user name
#   PASSWD     : user password
#   SECUREFLAG : flag to use smtp or secure ftp 'smtps'
#
# Execute script example"
#./test_export_publischer.sh FTP testuser ftp ftpserver.com 21 /subdir true
#./test_export_publisher.sh  EMAIL testuser testuserpPasswd mailServer.com 456 LOLO false
#===============================================================================

#is_empty_or_none:
# check if parameter string $1 is empty or none
#
#return:
#      1 string is not none and have length >0
#      0 string have a zero length or none
is_empty_or_none(){
    if [ "$1" = "none" ] || [ -z "$1" ] ; then
        return 0
    else
        return 1
    fi
}

#test_ftp:
#       check if the given ftp server is reachable and we have the permission 
#       to list the given directory.
# return 0 on success and 1 otherwise
#
#parameters: TYPE USERNAME PASSWORD SERVER PORT SUBDIR SECUREFLAG
#e.g Username Password Server 21 /tmp/ true
#             !!!!!server and port are mandatory!!!!!
test_ftp(){
    # check FTP parameter
    if [ $#  -ne 6 ] ; then
        echo "ERROR: Parameters missing in cmdline $@."
        return 1
    fi
    USERNAME="$1"
    PASSWD="$2" 
    SERVER="$3"
    PORT="$4"
    SUBDIR="$5"
    SECURE="$6"

    PROTOCOL="ftp"
    #CURLFTPPARAMETERS=--insecure --connect-timeout 10 -user $USERNAME:$PASSWD --url PROTOCOL://$SERVER:$PORT/$SUBDIR/
    CURLFTPPARAMETERS=" --insecure --connect-timeout 10 --silent --show-error"
    
    # check for secure ftp flag
    if [ "x$SECURE" = "xtrue" ] ; then
        PROTOCOL="sftp"
    fi
    if is_empty_or_none "$USERNAME" || is_empty_or_none "$PASSWD " ; then
        if [ "x$SECURE" = "xtrue" ] ; then
            echo "ERROR: Username or password required for sFTP! "
            return 1
        fi
    else
        CURLFTPPARAMETERS="$CURLFTPPARAMETERS --user $USERNAME:$PASSWD "
    fi

    # check mandatory parameter ftp-server
    if is_empty_or_none "$SERVER" ; then
        echo "ERROR: ftp-server required."
        return 1
    fi
    
    #check mandatory parameter port
    if is_empty_or_none "$PORT"; then
        echo "ERROR: ftp port required."
        return 1
    fi
    # get weblogger SN
    logger_sn=$(fw_printenv | grep "sn=" | cut -d"=" -f2)
    if [ -z "${logger_sn}" ]
    then
        logger_sn="UNKNOWN"
    fi
    
    CURLFTPPARAMETERS="$CURLFTPPARAMETERS"" --url $PROTOCOL://$SERVER:$PORT/"
    
    #check subdirectory
    if ! is_empty_or_none "$SUBDIR" ; then
        CURLFTPPARAMETERS="$CURLFTPPARAMETERS""$SUBDIR/"
    fi

    # check if we can write a file to the server
    {
    printf "Test file to be uploaded via ftp: Weblog (SN=%s)\n" "${logger_sn}";
    } > /tmp/Testftp.txt

    # execute curl to copy file to ftp directory 
    if ! result=$(curl -T /tmp/Testftp.txt $CURLFTPPARAMETERS 2>&1)
    then
        echo "${result}"
        echo "ERROR: Failed to connect to FTP-Server ${SERVER}."
        rm /tmp/Testftp.txt
        return 1
    fi
    rm /tmp/Testftp.txt
    #Remove created file on remote ftp-server
    if ! result=$(curl $CURLFTPPARAMETERS -Q "rm Testftp.txt" 2>&1)
    then
        echo "${result}"
        echo "ERROR: Failed to remove Test file Testftp.txt from FTP-Server ${SERVER}."
        return 1
    fi
    
    return 0
}

#test_mail:
# send test mail to all addresses given in TOADDRESS=$1
#
# return 0 on success and 1 otherwise
#parameters: 
# TOADDRESS FROMADDRESS  SERVER USERNAME PASSWD SECUREFLAG
test_mail() {
    
    # check EMAIL parameter
    if [ $# -ne 6 ] ; then
        echo "ERROR: Parameters missing ($#/6)in cmdline <$*>"
        return 1
    fi

    TOADDRESS=$1
    FROMADDRESS=$2
    SERVER=$3
    USERNAME=$4
    PASSWD=$5
    SECURE=$6

    CURLMAILPARAMETERS="--insecure --ssl-reqd --insecure --connect-timeout 10 --silent --show-error --url"
    # check for secure mail flag 
    if [ "x$SECURE" = "xtrue" ] ; then
        PROTOCOL="smtps"
    else 
        PROTOCOL="smtp"
    fi
    
    if is_empty_or_none "$SERVER" ; then
        echo "ERROR: Email server required."
        return 1
    fi
    
    CURLMAILPARAMETERS="$CURLMAILPARAMETERS $PROTOCOL://$SERVER "
    
    #check recipient
    if is_empty_or_none "$TOADDRESS"; then
        echo "ERROR: recipient (TO mail address) required."
        return 1
    else
        RECIPIENTS=" --mail-rcpt $TOADDRESS "
        RECIPIENTS=$(echo "$RECIPIENTS"|sed 's/;/ --mail-rcpt /g')
    fi

    CURLMAILPARAMETERS="$CURLMAILPARAMETERS $RECIPIENTS "

    #check USERNAME amd PASSWORD
    if [ "x$SECURE" = "xtrue" ] ; then
        if is_empty_or_none "$USERNAME" || is_empty_or_none "$PASSWD" ; then
            echo "ERROR: SMTP-User or SMTP-Password required for smtps! "
            return 1
        else
            CURLMAILPARAMETERS="$CURLMAILPARAMETERS --user $USERNAME:$PASSWD "
        fi
    else
        if is_empty_or_none "$USERNAME" || is_empty_or_none "$PASSWD" ; then
            CURLMAILPARAMETERS="$CURLMAILPARAMETERS" " --user '':'' "  
        else
            CURLMAILPARAMETERS="$CURLMAILPARAMETERS --user $USERNAME:$PASSWD "  
        fi
    fi

    #check from address

    if ! is_empty_or_none "$FROMADDRESS"; then
        CURLMAILPARAMETERS="$CURLMAILPARAMETERS --mail-from $FROMADDRESS "
    fi

    logger_sn=$(fw_printenv | grep "sn=" | cut -d"=" -f2)
    if [ -z "${logger_sn}" ]
    then
        logger_sn="UNKNOWN"
    fi
    {
	echo "From: $FROMADDRESS";
	echo "To: \"Weblog Test mail\" <anyone@weblogtest.com>"; 
    echo -e "Subject: Weblog Export-Test Email\r";
	echo -e "\r";
    echo -e "This is an EMail submitted as a test from a Weblog250 (SN=${logger_sn}).\r";
    } > /tmp/Testmail.txt
    if ! result=$(curl $CURLMAILPARAMETERS  --upload-file /tmp/Testmail.txt 2>&1)
    then
        rm -f /tmp/Testmail.txt
        echo "${result}"
        echo "ERROR: Sending EMail to <$TOADDRESS> failed."
        return 1
    fi

    rm -f /tmp/Testmail.txt
    return 0
}

####### main #######
if [ "$#" -gt 0 ] ;then
    SCHEDULERTYPE=$1
    shift
else
    echo "ERROR: Scheduler Type (FTP or EMAIL) is empty"
    exit 1
fi

case "$SCHEDULERTYPE" in
    "FTP")
        if ! test_ftp "$@" ; then
            exit 1
        else
            echo "SUCCESS: FTP-server works."
            exit 0
        fi
    ;;
    "EMAIL")
        if ! test_mail "$@" ; then
            exit 1
        else
            echo "SUCCESS: Testemail was sent. Please check your mailbox."
            exit 0
        fi
    ;;
    *)
        echo "ERROR: Unknown option <${SCHEDULERTYPE}>; expected FTP or EMAIL."
        exit 1
    ;;
esac
